#
# SPDX-FileCopyrightText: Copyright 2025 Siemens AG
# SPDX-License-Identifier: Apache-2.0
#
import sys
from typing import Optional
from licenselynx.license_object import LicenseObject
from licenselynx.license_map_singleton import _LicenseMapSingleton


class LicenseLynx:

    @staticmethod
    def map(license_name: str, risky: bool = False) -> Optional[LicenseObject]:
        """
        Maps license name to the canonical license identifier
        :param license_name: string of a license name
        :param risky: enable risky mappings
        :return: LicenseObject with the canonical license identifier and source, None if no license is found,
        or throws an exception if a runtime error occurs
        """
        try:

            instance = _LicenseMapSingleton()

            license_object: Optional[LicenseObject] = instance.merged_data.stable_map.get(license_name)

            if not license_object and risky:
                license_object = instance.merged_data.risky_map.get(license_name)

            if not license_object:
                return None

            return license_object
        except Exception as e:
            raise e.with_traceback(sys.exc_info()[2])



#
# SPDX-FileCopyrightText: Copyright 2025 Siemens AG
# SPDX-License-Identifier: Apache-2.0
#
import json
import sys
from importlib import resources
from threading import Lock
from licenselynx.license_map import _LicenseMap
from licenselynx.license_object import LicenseObject


class _Singleton(type):
    _instances: dict[type, type] = {}
    _lock: Lock = Lock()

    def __call__(cls, *args, **kwargs):
        with cls._lock:
            if cls not in cls._instances:
                instance = super().__call__(*args, **kwargs)
                cls._instances[cls] = instance
        return cls._instances[cls]


class _LicenseMapSingleton(metaclass=_Singleton):
    def __init__(self):
        self._file_path = resources.files("licenselynx.resources").joinpath("merged_data.json")
        try:
            with self._file_path.open() as file:
                data = json.load(file)

                stable_map = {}
                for key, value in data["stable_map"].items():
                    stable_map[key] = LicenseObject(**value)
                risky_map = {}
                for key, value in data["risky_map"].items():
                    risky_map[key] = LicenseObject(**value)

                self._merged_data = _LicenseMap(stable_map, risky_map)
        except Exception as e:
            raise e.with_traceback(sys.exc_info()[2])

    @property
    def merged_data(self) -> _LicenseMap:
        return self._merged_data



#
# SPDX-FileCopyrightText: Copyright 2025 Siemens AG
# SPDX-License-Identifier: Apache-2.0
#
from dataclasses import dataclass

from licenselynx.license_object import LicenseObject


@dataclass
class _LicenseMap(object):
    stable_map: dict[str, LicenseObject]
    risky_map: dict[str, LicenseObject]



#
# SPDX-FileCopyrightText: Copyright 2025 Siemens AG
# SPDX-License-Identifier: Apache-2.0
#
from dataclasses import dataclass


@dataclass
class LicenseObject(object):
    canonical: str
    src: str
